"""Public request decoder."""
from fred_api.public_request.common_types_pb2 import (
    CannotBeProcessed as CannotBeProcessedMsg,
    ObjectId as ObjectIdMsg,
    ObjectType as ObjectTypeMsg,
    OnStatusAction as OnStatusActionMsg,
    PublicRequest as PublicRequestMsg,
    PublicRequestDoesNotExist as PublicRequestDoesNotExistMsg,
    Status as StatusMsg,
)
from frgal import GrpcDecoder

from .common import ObjectId, PublicRequest
from .exceptions import CannotBeProcessed, PublicRequestDoesNotExist


class PublicRequestDecoder(GrpcDecoder):
    """Public request message decoder."""

    decode_empty_string_none = False

    def __init__(self) -> None:
        super().__init__()
        self.set_exception_decoder(CannotBeProcessedMsg, CannotBeProcessed)
        self.set_exception_decoder(PublicRequestDoesNotExistMsg, PublicRequestDoesNotExist)
        self.set_decoder(PublicRequestMsg, self._decode_public_request)
        self.set_decoder(ObjectIdMsg, self._decode_object_id)

    def _decode_object_id(self, value: ObjectIdMsg) -> ObjectId:
        """Decode object identifier."""
        kwargs = self._decode_message(value)
        # enumeration field with value 0 (meaning 'unknown') yields to validation error
        kwargs["type"] = ObjectTypeMsg.Enum.Name(value.type)
        return ObjectId(**kwargs)

    def _decode_public_request(self, value: PublicRequestMsg) -> PublicRequest:
        """Decode public request information."""
        kwargs = self._decode_message(value)
        # avoid empty strings
        if not kwargs["registrar_id"]:
            kwargs.pop("registrar_id")
        if not kwargs["response_email"]:
            kwargs.pop("response_email")
        if not kwargs["creation_id"]:
            kwargs.pop("creation_id")
        if not kwargs["resolving_id"]:
            kwargs.pop("resolving_id")
        # enumeration fields with value 0 (meaning 'unknown') yield to validation error
        kwargs["status"] = StatusMsg.Enum.Name(value.status)
        kwargs["on_status_action"] = OnStatusActionMsg.Enum.Name(value.on_status_action)
        return PublicRequest(**kwargs)
