import warnings
from unittest import TestCase

from pydantic import BaseModel

from prequel.common import BaseId, _BaseModel
from prequel.exceptions import ExtraArgumentWarning


class BaseModelTest(TestCase):
    def test_warning(self):
        class TestModel(_BaseModel):
            name: str

        with warnings.catch_warnings(record=True) as captured:
            warnings.simplefilter("always")

            TestModel(name="Kryten", id="2X4B", ship="Red Dwarf")

        self.assertEqual(len(captured), 2)
        self.assertIsInstance(captured[0].message, ExtraArgumentWarning)
        self.assertIsInstance(captured[1].message, ExtraArgumentWarning)


class BaseIdTest(TestCase):
    def test_validation(self):
        class Model(BaseModel):
            uid: BaseId

        self.assertEqual(Model(uid="007").uid, BaseId("007"))

    def test_repr(self):
        self.assertEqual(repr(BaseId("007")), "BaseId('007')")
